/************************************************************************
 *  M_ALL.C								*
 *				Befehle fr MINI51			*
 *				==================			*
 *									*
 *  06.06.96	Bodo Wenzel	V1.0, bernahme aus X51			*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <absacc.h>
#include <reg51.h>
#include <stdio.h>

#include "utility.h"
#include "mini51.h"
#include "command.h"
#include "m_all.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_HE=ID_NONE+1, ID_RS, ID_FM, ID_DL,
	ID_XW, ID_XR, ID_XO, ID_XI, ID_XP, ID_XT,
	ID_PO, ID_PI, ID_0I, ID_0S };

/* === Funktionsprototypen ============================================ */

static entry *m_help(bit doit);
static entry *m_reset(bit doit);
static entry *m_frequ(bit doit);
static entry *m_delay(bit doit);
static entry *m_x_write(bit doit);
static entry *m_x_read(bit doit);
static entry *m_x_out(bit doit);
static entry *m_x_inp(bit doit);
static entry *m_x_prep(bit doit);
static entry *m_x_test(bit doit);
static bit mxt_abort(uint a,uchar d);
static entry *m_port_out(bit doit);
static entry *m_port_inp(bit doit);
static entry *m_off_idle(bit doit);
static entry *m_off_stop(bit doit);

/* === Funktionen ===================================================== */

/* --- Kommandoverzweigungen ------------------------------------------ */

bit m_all_look(void) {
/*------------------*/
if ((cmd_entry=m_help(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_reset(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_frequ(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_delay(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_write(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_read(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_prep(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_test(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_port_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_port_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_idle(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_stop(FALSE))!=0)
  return(TRUE);
return(FALSE);
}

void m_all_doit(void) {
/*-------------------*/
uchar	id;

switch (id=*cmd_act++) {
case ID_HE:
  m_help(TRUE);
  break;
case ID_RS:
  m_reset(TRUE);
  break;
case ID_FM:
  m_frequ(TRUE);
  break;
case ID_DL:
  m_delay(TRUE);
  break;
case ID_XW:
  m_x_write(TRUE);
  break;
case ID_XR:
  m_x_read(TRUE);
  break;
case ID_XO:
  m_x_out(TRUE);
  break;
case ID_XI:
  m_x_inp(TRUE);
  break;
case ID_XP:
  m_x_prep(TRUE);
  break;
case ID_XT:
  m_x_test(TRUE);
  break;
case ID_PO:
  m_port_out(TRUE);
  break;
case ID_PI:
  m_port_inp(TRUE);
  break;
case ID_0I:
  m_off_idle(TRUE);
  break;
case ID_0S:
  m_off_stop(TRUE);
  break;
default:
  break;
  }
}

/* --- Hilfe ausgeben ------------------------------------------------- */

static entry *m_help(bit doit) {
/*----------------------------*/
entry	code	e = { ID_HE, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='H' && cmd_c2=='E')
    return(&e);
  else
    return(0);
  }
rs232_puts("\n[*][cmd [[v],v][. cmd...]]"
           "\nHE RS FMb DLt "
           "XWa,d XRa XOa,d XIa XPb,e XTb,e "
           "POp,d PIp 0I 0S");
return(0);
}

/* --- Allgemeine Befehle --------------------------------------------- */

static entry *m_reset(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_RS, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='R' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
rs232_puts("\n\xFF");
EA=FALSE;
(* (void (*)()) 0U) ();
return(0);
}

static entry *m_frequ(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_FM, T_DEZ, 50, 19200, T_NONE, 0, 0 };
long		f;

if (!doit) {
  if (cmd_c1=='F' && cmd_c2=='M')
    return(&e);
  else
    return(0);
  }
f=*((uint *)cmd_act);
cmd_act+=2;
f*=bit_ticks;
f+=5000L;
putchar('\n');
rs232_putd(f/1000000L);
putchar('.');
f%=1000000L;
rs232_putd(f/100000L);
f%=100000L;
rs232_putd(f/10000L);
return(0);
}

static entry *m_delay(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_DL, T_DEZ, 0, 65535, T_NONE, 0, 0 };
uint		t;

if (!doit) {
  if (cmd_c1=='D' && cmd_c2=='L')
    return(&e);
  else
    return(0);
  }
t=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=(-(int)t)>>8;
TL0=(uchar)(-(int)t);
TF0=FALSE;
TR0=TRUE;
while (!TF0);
return(0);
}

/* --- Externen Speicher bearbeiten ----------------------------------- */

static entry *m_x_write(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_XW, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='W')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_read(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XR, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='R')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_out(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XO, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_inp(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XI, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_prep(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XP, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='P')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  XBYTE[a]=d;
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
XBYTE[a]=d;
return(0);
}

static entry *m_x_test(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XT, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='T')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  if (XBYTE[a]!=d)
    if (mxt_abort(a,d))
      return(0);
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
if (XBYTE[a]!=d)
  mxt_abort(a,d);
return(0);
}

static bit mxt_abort(uint a,uchar d) {
/*----------------------------------*/
putchar('\n');
rs232_putx(a,TRUE);
putchar(':');
rs232_putx(d,FALSE);
putchar('>');
rs232_putx(XBYTE[a],FALSE);
rs232_puts(" key (ESC)");
return(_getkey()=='\x1B');
}

/* --- 80C51-Befehle -------------------------------------------------- */

static entry *m_port_out(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PO, T_DEZ, 0, 3, T_HEX, 0x00, 0xFF };
uint		p,d;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
switch (p) {
case 0:
  P0=(uchar)d;
  break;
case 1:
  P1=(uchar)d;
  break;
case 2:
  P2=(uchar)d;
  break;
case 3:
  P3=(uchar)d;
  break;
default:
  break;
  }
return(0);
}

static entry *m_port_inp(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PI, T_DEZ, 0, 3, T_NONE, 0, 0 };
uint		p;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
switch (p) {
case 0:
  rs232_putx(P0,FALSE);
  break;
case 1:
  rs232_putx(P1,FALSE);
  break;
case 2:
  rs232_putx(P2,FALSE);
  break;
case 3:
  rs232_putx(P3,FALSE);
  break;
default:
  break;
  }
return(0);
}

static entry *m_off_idle(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0I, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
rs232_puts("Idle (key)");
rs232_int_on();
PCON|=0x01;
rs232_int_off();
return(0);
}

static entry *m_off_stop(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0S, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
rs232_puts("Down\xFF");
PCON|=0x02;
return(0);
}

/* === Ende =========================================================== */
