/************************************************************************
 *  M_ALL.C								*
 *			Befehle fr alle Prozessoren fr MON51		*
 *			======================================		*
 *									*
 *  22.06.93	Bodo Wenzel	V1.0, Erstellung			*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <absacc.h>
#include <reg51.h>
#include <stdio.h>

#include "utility.h"
#include "mon51.h"
#include "command.h"
#include "m_all.h"
#include "sfr.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_HE=ID_NONE+1, ID_RS, ID_FM,
	ID_SO, ID_SI, ID_SS, ID_ST,
	ID_XW, ID_XR, ID_XO, ID_XI, ID_XP, ID_XT,
	ID_MH, ID_ML, ID_OH, ID_OL };

/* === Funktionsprototypen ============================================ */

static entry *m_help(bit doit);
static entry *m_reset(bit doit);
static entry *m_frequ(bit doit);
static entry *m_sfr_out(bit doit);
static entry *m_sfr_inp(bit doit);
static entry *m_sfrb_set(bit doit);
static entry *m_sfrb_test(bit doit);
static entry *m_x_write(bit doit);
static entry *m_x_read(bit doit);
static entry *m_x_out(bit doit);
static entry *m_x_inp(bit doit);
static entry *m_x_prep(bit doit);
static entry *m_x_test(bit doit);
static bit mxt_abort(uint a,uchar d);
static entry *m_meas_h(bit doit);
static entry *m_meas_l(bit doit);
static entry *m_out_h(bit doit);
static entry *m_out_l(bit doit);

/* === Funktionen ===================================================== */

bit m_all_look(void) {
/*------------------*/
if ((cmd_entry=m_help(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_reset(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_frequ(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfr_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfr_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfrb_set(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfrb_test(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_write(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_read(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_prep(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_test(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_meas_h(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_meas_l(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_out_h(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_out_l(FALSE))!=0)
  return(TRUE);
return(FALSE);
}

void m_all_doit(void) {
/*-------------------*/
uchar	id;

switch (id=*cmd_act++) {
case ID_HE:
  m_help(TRUE);
  break;
case ID_RS:
  m_reset(TRUE);
  break;
case ID_FM:
  m_frequ(TRUE);
  break;
case ID_SO:
  m_sfr_out(TRUE);
  break;
case ID_SI:
  m_sfr_inp(TRUE);
  break;
case ID_SS:
  m_sfrb_set(TRUE);
  break;
case ID_ST:
  m_sfrb_test(TRUE);
  break;
case ID_XW:
  m_x_write(TRUE);
  break;
case ID_XR:
  m_x_read(TRUE);
  break;
case ID_XO:
  m_x_out(TRUE);
  break;
case ID_XI:
  m_x_inp(TRUE);
  break;
case ID_XP:
  m_x_prep(TRUE);
  break;
case ID_XT:
  m_x_test(TRUE);
  break;
case ID_MH:
  m_meas_h(TRUE);
  break;
case ID_ML:
  m_meas_l(TRUE);
  break;
case ID_OH:
  m_out_h(TRUE);
  break;
case ID_OL:
  m_out_l(TRUE);
  break;
default:
  break;
  }
}

/* --- Hilfe ausgeben ------------------------------------------------- */

static entry *m_help(bit doit) {
/*----------------------------*/
entry	code	e = { ID_HE, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='H' && cmd_c2=='E')
    return(&e);
  else
    return(0);
  }
rs232_puts("\nEmpty repeats, asterisk loops, ESC aborts"
           "\n\"cmd [[v],v][. cmd...]\""
           "\nHE    this"
           "\nRS    reset"
           "\nFMb   xtalfreq"
           "\nSOa,d wr sfrbyte"
           "\nSIa   rd sfrbyte"
           "\nSSa,d set sfrbit"
           "\nSTa   tst sfrbit"
           "\nXWa,d wr++ xdata"
           "\nXRa   rd++ xdata"
           "\nXOa,d wr xdata"
           "\nXIa   rd xdata"
           "\nXPb,e prep xdata"
           "\nXTb,e test xdata"
           "\nMHa   high pulse"
           "\nMLa   low pulse"
           "\nOHa,t high pulse out"
           "\nOLa,t low pulse out");
return(0);
}

/* --- Allgemeine Befehle --------------------------------------------- */

static entry *m_reset(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_RS, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='R' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
putchar('\n');
putchar('\xFF');
EA=FALSE;
(* (void (*)()) 0U) ();
return(0);
}

static entry *m_frequ(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_FM, T_DEZ, 50, 19200, T_NONE, 0, 0 };
long		f;

if (!doit) {
  if (cmd_c1=='F' && cmd_c2=='M')
    return(&e);
  else
    return(0);
  }
f=*((uint *)cmd_act);
cmd_act+=2;
f*=bit_ticks;
f+=500L;
putchar('\n');
rs232_putd(f/1000L);
return(0);
}

/* --- Spezial-Funktion-Register bearbeiten --------------------------- */

static entry *m_sfr_out(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_SO, T_HEX, 0x80, 0xFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
sfr_out((uchar)a,(uchar)d);
return(0);
}

static entry *m_sfr_inp(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_SI, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putx(sfr_inp((uchar)a),FALSE);
return(0);
}

static entry *m_sfrb_set(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_SS, T_HEX, 0x80, 0xFF, T_DEZ, 0, 1 };
uint		a,d;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
sfr_set((uchar)a,(bit)d);
return(0);
}

static entry *m_sfrb_test(bit doit) {
/*---------------------------------*/
entry	code	e = { ID_ST, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='T')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putd(sfr_test((uchar)a));
return(0);
}

/* --- Externen Speicher bearbeiten ----------------------------------- */

static entry *m_x_write(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_XW, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='W')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_read(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XR, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='R')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_out(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XO, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_inp(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XI, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_prep(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XP, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='P')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  XBYTE[a]=d;
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
XBYTE[a]=d;
return(0);
}

static entry *m_x_test(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XT, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='T')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  if (XBYTE[a]!=d)
    if (mxt_abort(a,d))
      return(0);
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
if (XBYTE[a]!=d)
  mxt_abort(a,d);
return(0);
}

static bit mxt_abort(uint a,uchar d) {
/*----------------------------------*/
putchar('\n');
rs232_putx(a,TRUE);
putchar(':');
rs232_putx(d,FALSE);
putchar('>');
rs232_putx(XBYTE[a],FALSE);
rs232_puts(" (any key/ESC)");
return(_getkey()=='\x1B');
}

/* --- Peripherie-Tests ----------------------------------------------- */

static entry *m_meas_h(bit doit) {
/*------------------------------*/
entry	code	e = { ID_MH, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='M' && cmd_c2=='H')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=0;
TL0=0;
TF0=FALSE;
while (!sfr_test((uchar)a))
  if (RI)
    break;
TR0=TRUE;
while (sfr_test((uchar)a))
  if (RI)
    break;
TR0=FALSE;
putchar('\n');
if (!TF0) {
  a=((uint)TH0<<8) | TL0;
  rs232_putd(a);
} else
  rs232_puts("over");
if (RI)
  _getkey();
return(0);
}

static entry *m_meas_l(bit doit) {
/*------------------------------*/
entry	code	e = { ID_ML, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='M' && cmd_c2=='L')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=0;
TL0=0;
TF0=FALSE;
while (sfr_test((uchar)a))
  if (RI)
    break;
TR0=TRUE;
while (!sfr_test((uchar)a))
  if (RI)
    break;
TR0=FALSE;
putchar('\n');
if (!TF0) {
  a=((uint)TH0<<8) | TL0;
  rs232_putd(a);
} else
  rs232_puts("over");
if (RI)
  _getkey();
return(0);
}

static entry *m_out_h(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_OH, T_HEX, 0x80, 0xFF, T_DEZ, 0, 65535 };
uint		a,t;

if (!doit) {
  if (cmd_c1=='O' && cmd_c2=='H')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
t=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=(-(int)t)>>8;
TL0=(uchar)(-(int)t);
TF0=FALSE;
TR0=TRUE;
sfr_set((uchar)a,1);
while (!TF0);
sfr_set((uchar)a,0);
return(0);
}

static entry *m_out_l(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_OL, T_HEX, 0x80, 0xFF, T_DEZ, 0, 65535 };
uint		a,t;

if (!doit) {
  if (cmd_c1=='O' && cmd_c2=='L')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
t=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=(-(int)t)>>8;
TL0=(uchar)(-(int)t);
TF0=FALSE;
TR0=TRUE;
sfr_set((uchar)a,0);
while (!TF0);
sfr_set((uchar)a,1);
return(0);
}

/* === Ende =========================================================== */
