/************************************************************************
 *  RAMTEST.C								*
 *			Speicher-Testprogramm fr 8051			*
 *			==============================			*
 *									*
 *  13.12.95	Bodo Wenzel	V1.0, Erstellung fr 256er ab 0000	*
 *  15.12.95	Bodo Wenzel	V1.1, Automatische Speicherermittlung	*
 *  30.11.98	Bodo Wenzel	V1.2, Fehler in test_a() beseitigt	*
 ************************************************************************/

#pragma	CODE SYMBOLS SMALL DEBUG OBJECTEXTEND

#define	VERSION	"V1.2"

#include <absacc.h>
#include <ctype.h>
#include <reg51.h>
#include <stdio.h>

/* === Utility ======================================================== */

#define	FALSE	0
#define	TRUE	(!FALSE)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;

/* === RS232 ========================================================== */

void rs232_init(void) {
/*-------------------*/
uint	t;

SCON=0x58;			/* 8-N-1, Empfang freigeben		*/
PCON|=0x80;			/* doppelte Baudrate			*/

for (;;) {
  TR1=FALSE;
  TMOD&=0x0F;
  TMOD|=0x10;
  TH1=0;
  TL1=0;			/* Timer 1 stoppen und lschen		*/

  while (RXD);
  TR1=TRUE;			/* Bei fallender Flanke starten		*/
  while (!RXD);
  TR1=FALSE;
  t=((uint)TH1<<8) | TL1;	/* Bei steigender Flanke messen		*/

  if (t<8)
    continue;
  if (t>0xFF7)
    continue;			/* Unpassende Bitlnge			*/

  TH1=(-(int)t*8)>>8;
  TL1=(uchar)(-(int)t*8);
  TF1=FALSE;
  TR1=TRUE;
  while (!TF1);
  TR1=FALSE;			/* Rest des Datenbytes abwarten		*/

  TMOD&=0x0F;
  TMOD|=0x20;
  TH1=-(uchar)((t+8)/16);
  TR1=TRUE;			/* als Baudraten-Timer setzen		*/

  RI=FALSE;
  if (_getkey()=='\r')
    break;			/* auf CR warten			*/
  }

RI=FALSE;			/* Empfnger bereit			*/
TI=TRUE;			/* Sender bereit			*/
}

/* === Laufanzeige ==================================================== */

char	code	show_s[]="/\b-\b\\\b|\b";
char	code	*show_c;

void show_set(void) {
/*-----------------*/
putchar('\r');
show_c=show_s;			/* Anzeige vorbereiten			*/
}

void show(void) {
/*-------------*/
if (TI) {			/* nur wenn letztes Zeichen weg		*/
  if (*show_c=='\0')
    show_c=show_s;
  putchar(*show_c++);		/* zyklisch anzeigen			*/
  }
}

/* === Hilfsfunktionen ================================================ */

uint		start,end;		/* Zu testendes RAM		*/
long		size;

bit auto_start(void) {
/*------------------*/
start=0x0000;
do {
  show();
  XBYTE[start]=0x5A;
  if (XBYTE[start]==0x5A) {
    printf("\rStart %04XH\n",start);
    return(TRUE);
    }
} while (++start!=0x0000);
return(FALSE);
}

bit auto_size(void) {
/*-----------------*/
XBYTE[start]=0x5A;
for (size=0x0001; size!=0x10000; size<<=1) {
  show();
  XBYTE[start+size]=0xA5;
  if (XBYTE[start]==0xA5)		/* Spiegelung suchen		*/
    break;
  if (XBYTE[start+size]!=0xA5)		/* Ende suchen			*/
    break;
  }
if (size<0x100)				/* Mindestgre 256 Byte	*/
  return(FALSE);
printf("\rSize %5lXH\n",size);
return(TRUE);
}

long	bdata	rnd_r;			/* Zufallszahl			*/

sbit	rnd_24	=rnd_r^0;		/* 0x01000000	*/
sbit	rnd_19	=rnd_r^11;		/* 0x00080000	*/
sbit	rnd_13	=rnd_r^21;		/* 0x00002000	*/
sbit	rnd_6	=rnd_r^30;		/* 0x00000040	*/
sbit	rnd_0	=rnd_r^24;		/* 0x00000001	*/

uchar rnd(void) {
/*-------------*/
rnd_r<<=1;
rnd_0=1;
if (rnd_24)
  rnd_0^=1;
if (rnd_19)
  rnd_0^=1;
if (rnd_13)
  rnd_0^=1;
if (rnd_6)
  rnd_0^=1;
return((uchar)rnd_r);
}

void check(uint a,uchar w) {
/*------------------------*/
uchar	r;

r=XBYTE[a];				/* Byte lesen			*/
if (r!=w) {
  show_set();
  printf("\a%02bX -> %04X -> %02bX\n",w,a,r);
  _getkey();				/* Fehler melden		*/
  }
}

/* === Test-Funktionen ================================================ */

void test_d(void) {			/* === Datenbus =============== */
/*---------------*/
uint	a;
uchar	w;

w=0x00;
do {
  show();
  XBYTE[start]=w;
  check(start,w);
  w++;
} while (w!=0x00);			/* alle mglichen Werte		*/

a=start;
do {
  show();
  XBYTE[a]=0x55;
  check(a++,0x55);
} while (a!=end+1);			/* Muster 55H			*/

a=start;
do {
  show();
  XBYTE[a]=0xAA;
  check(a++,0xAA);
} while (a!=end+1);			/* Muster AAH			*/

show_set();
}

void test_a(void) {			/* === Adrebus =============== */
/*---------------*/
struct	{
	uint	a;
	uchar	d;
	}	code	p[17]={
	{ 0x0001, 0x01 },	{ 0x0002, 0x23 },
	{ 0x0004, 0x45 },	{ 0x0008, 0x67 },
	{ 0x0010, 0x89 },	{ 0x0020, 0xAB },
	{ 0x0040, 0xCD },	{ 0x0080, 0xEF },
	{ 0x0100, 0x12 },	{ 0x0200, 0x34 },
	{ 0x0400, 0x56 },	{ 0x0800, 0x78 },
	{ 0x1000, 0x9A },	{ 0x2000, 0xBC },
	{ 0x4000, 0xDE },	{ 0x8000, 0xF0 },
	{ 0x0000, 0xFF } };
uint	a;
uchar	i,j;

a=start;
do {
  show();
  XBYTE[a++]=p[16].d;
} while (a!=end+1);
for (i=0; start+p[i].a!=end+1 && i<17; i++) {
  show();
  XBYTE[start+p[i].a]=p[i].d;
  XBYTE[end-p[i].a]=~p[i].d;
  }					/* Speicher vorbereiten		*/
j=i-1;

i=0;
a=start;
do {
  show();
  if (a==start+p[i].a)
    check(a++,p[i++].d);
  else if (a==end-p[j].a)
    check(a++,~p[j--].d);
  else
    check(a++,p[16].d);
} while (a!=end+1);			/* Speicher berprfen		*/

show_set();
}

void test_p(void) {			/* === Muster ================= */
/*---------------*/
uint	a;

a=start;
rnd_r=0xDEADC0DEL;
do {
  show();
  XBYTE[a++]=rnd();
} while (a!=end+1);			/* Speicher vorbereiten		*/

a=start;
rnd_r=0xDEADC0DEL;
do {
  show();
  check(a++,rnd());
} while (a!=end+1);			/* Speicher testen		*/

show_set();
}

void t_b_page(uint s,uint e,uchar wn) {	/* --- eine Seite Peaktest ---- */
/*-----------------------------------*/
uint	aw,ar;
uchar	w;

show_set();
printf("%04X..%04X ",s,e);

w=0x00;
aw=s;
do {
  w<<=1;				/* Bit zyklisch rotieren	*/
  if (w==0x00)
    w=0x01;
  XBYTE[aw]=wn^w;
  for (ar=s; ar!=aw; ar++) {
    show();
    check(ar,wn);
    }
  check(ar++,wn^w);
  for ( ; ar!=e+1; ar++) {
    show();
    check(ar,wn);
    }
  XBYTE[aw++]=wn;
} while (aw!=e+1);
}

void test_b(bit v) {			/* === Peak/Valley-Test ======= */
/*----------------*/
uint	a;
uchar	w;

w=v ? 0x00:0xFF;			/* Testwert vorbereiten		*/

a=start;
do {
  show();
  XBYTE[a++]=w;
} while (a!=end+1);			/* Speicher vorbereiten		*/

a=start;
do {
  t_b_page(a,a+0xFF,w);
  a+=0x100;
} while (a!=end+1);			/* seitenweise testen		*/

show_set();
}

/* === M A I N ======================================================== */

void main(void) {
/*-------------*/
char	l[8];
long	i;

rs232_init();				/* Hardware initialisieren	*/

show_set();				/* Software initialisieren	*/

puts("RAMTEST X51 " VERSION "/" __DATE__ " (C)1995 Bodo Wenzel");

printf("Enter start address (hex, empty=auto): ");
gets(l,7);
if (sscanf(l,"%x",&start)!=1) {
  puts("Auto start...");
  if (!auto_start()) {			/* Anfang setzen		*/
    puts("\rNo RAM found, system halted!");
    for (;;);
    }
  }
show_set();

printf("Enter size in bytes (hex, empty=auto): ");
gets(l,7);
if (sscanf(l,"%lx",&size)!=1 || size<0x100) {
  puts("Auto size...");
  if (!auto_size()) {			/* Lnge setzen			*/
    puts("\rNo RAM found, system halted!");
    for (;;);
    }
  }
show_set();

end=start+(uint)(size-1);		/* letzte Adresse		*/

for (i=1L; ; i++) {
  printf("Start %04XH, size %5lXH (%u Kbit), run %ld\n",
         start,size,(uint)(size/128),i);
  puts("Data bus...     ");
  test_d();
  puts("Address bus...  ");
  test_a();
  puts("Pattern...      ");
  test_p();
  puts("Bit shift 1...  ");
  test_b(1);
  puts("Bit shift 0...  ");
  test_b(0);
  puts("\aReady.          ");
  }
}

/* === Ende =========================================================== */
