/************************************************************************
 *  COMMAND.C								*
 *			Kommando-Verarbeitung fr X51			*
 *			=============================			*
 *									*
 *  22.06.93	Bodo Wenzel	V1.0, Erstellung			*
 *  02.12.98	Bodo Wenzel	V1.1, Abbruch erleichtert		*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <ctype.h>
#include <reg51.h>
#include <stdio.h>

#include "utility.h"
#define	extern
#include "command.h"
#undef extern
#include "m_all.h"
#include "rs232.h"

/* === Konstanten ===================================================== */

#define	MAXLINE	50

/* === Globale Variablen ============================================== */

char	idata	line[MAXLINE];

/* === Funktionsprototypen ============================================ */

static char cmd_first(void);
static char cmd_id(char c);
static char cmd_args(void);
static char cmd_value(bit dez,uint min,uint max,bit last);
static bit cmd_save(uchar d);

/* === Funktionen ===================================================== */

void cmd_init(void) {
/*-----------------*/
if (cmd_proc>=PMAX)
  cmd_proc=P8051;
line[0]=ID_NONE;
}

void cmd_doit(void) {
/*-----------------*/
char	c;

m_prompt();
putchar('>');
c=cmd_first();
cmd_act=line;
for (;;) {
  if (c=='\r')
    break;
  if ((cmd_c1=cmd_id(c))=='\0')
    break;
  if ((cmd_c2=cmd_id(_getkey()))=='\0')
    break;
  if (m_all_look()) {
    if (!cmd_save(cmd_entry->id))
      break;
    if ((c=cmd_args())=='\0')
      break;
    putchar(' ');
    *cmd_act=ID_NONE;
    if (c=='.')
      c=_getkey();
  } else {
    rs232_puts(" is a unknown command!");
    line[0]=ID_NONE;
    break;
    }
  }

if (line[0]!=ID_NONE)
  for (;;) {
    cmd_act=line;
    while (*cmd_act!=ID_NONE)
      m_all_doit();
    if (RI) {
      (void)_getkey();
      cmd_repeat=FALSE;
      }
    if (!cmd_repeat)
      break;
    }
putchar('\n');
}

/* --- Abfrage der einzelnen Befehlsteile ----------------------------- */

static char cmd_first(void) {
/*-------------------------*/
char	c;

cmd_repeat=FALSE;
for (;;) {
  if ((c=_getkey())=='\x1B')
    break;
  else if (c=='\r')
    break;
  else if (c=='*') {
    putchar(c);
    cmd_repeat=TRUE;
    c=_getkey();
    break;
  } else if (isalnum(c))
    break;
  }
return(c);
}

static char cmd_id(char c) {
/*------------------------*/
for (;;) {
  if (c=='\x1B') {
    line[0]=ID_NONE;
    putchar('.');
    return('\0');
  } else if (isalnum(c=toupper(c)))
    return(putchar(c));
  c=_getkey();
  }
}

static char cmd_args(void) {
/*------------------------*/
bit	last;
char	c;

last=cmd_entry->t2==T_NONE;
switch (cmd_entry->t1) {
case T_DEZ:
  if ((c=cmd_value(TRUE,cmd_entry->l1,cmd_entry->h1,last))=='\0')
    return('\0');
  break;
case T_HEX:
  if ((c=cmd_value(FALSE,cmd_entry->l1,cmd_entry->h1,last))=='\0')
    return('\0');
  break;
default:
  for (;;)
    if ((c=_getkey())=='\x1B') {
      line[0]=ID_NONE;
      putchar('.');
      return('\0');
    } else if (c=='\r')
      break;
    else if (c=='.') {
      putchar(c);
      break;
      }
  return(c);
  }

switch (cmd_entry->t2) {
case T_DEZ:
  if ((c=cmd_value(TRUE,cmd_entry->l2,cmd_entry->h2,TRUE))=='\0')
    return('\0');
  break;
case T_HEX:
  if ((c=cmd_value(FALSE,cmd_entry->l2,cmd_entry->h2,TRUE))=='\0')
    return('\0');
  break;
default:
  break;
  }
return(c);
}

static char cmd_value(bit dez,uint min,uint max,bit last) {
/*-------------------------------------------------------*/
uint	v;
char	c;

v=0;
for (;;) {
  if ((c=_getkey())=='\x1B') {
    line[0]=ID_NONE;
    putchar('.');
    return('\0');
    }
  if (last && c=='\r')
    break;
  if ((last && c=='.') || (!last && c==',')) {
    putchar(c);
    break;
    }
  if (dez) {
    if (!isdigit(c))
      continue;
    v=v*10+c-'0';
  } else {
    if (!isxdigit(c))
      continue;
    v=v*16+toint(c);
    }
  putchar(c);
  }

if (v<min || v>max) {
  line[0]=ID_NONE;
  rs232_puts(" is wrong");
  return('\0');
  }
cmd_save((uchar)(v>>8));
cmd_save((uchar)(v&0xFF));	/* ACHTUNG, Reihenfolge wie Keil-C	*/
return(c);
}

/* --- Speichern im Kommandopuffer ------------------------------------ */

static bit cmd_save(uchar d) {
/*--------------------------*/
if (cmd_act<line+MAXLINE-1) {
  *cmd_act++=d;
  return(TRUE);
  }
line[0]=ID_NONE;
rs232_puts(".\nLine too long!");
return(FALSE);
}

/* === Ende =========================================================== */
