/************************************************************************
 *  M80515.C								*
 *			Befehle fr Prozessor 80515 fr X51		*
 *			===================================		*
 *									*
 *  18.06.93	Bodo Wenzel	V1.0, Erstellung			*
 *  02.12.98	Bodo Wenzel	V1.1, Textausgabe krzer		*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <reg515.h>
#include <stdio.h>

#include "utility.h"
#include "command.h"
#include "m_all.h"
#include "m80515.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_PO=ID_SPECIAL, ID_PI, ID_AI };

/* === Funktionsprototypen ============================================ */

static entry *m_port_out(bit doit);
static entry *m_port_inp(bit doit);
static entry *m_an_inp(bit doit);

/* === Funktionen ===================================================== */

bit m80515_look(void) {
/*-------------------*/
if ((cmd_entry=m_port_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_port_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_an_inp(FALSE))!=0)
  return(TRUE);
return(FALSE);
}

void m80515_doit(uchar id) {
/*------------------------*/
switch (id) {
case ID_PO:
  m_port_out(TRUE);
  break;
case ID_PI:
  m_port_inp(TRUE);
  break;
case ID_AI:
  m_an_inp(TRUE);
  break;
default:
  break;
  }
}

void m80515_help(void) {
/*--------------------*/
rs232_puts("\nPOp,d write port (p=port, d=data)"
           "\nPIp   read port (p=port)"
           "\nAIa,r read analog (a=inp, r=ref)");
}

void m80515_init(void) {
/*--------------------*/
}

/* --- Port-Befehle --------------------------------------------------- */

static entry *m_port_out(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PO, T_DEZ, 0, 5, T_HEX, 0x00, 0xFF };
uint		p,d;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
switch (p) {
case 0:
  P0=(uchar)d;
  break;
case 1:
  P1=(uchar)d;
  break;
case 2:
  P2=(uchar)d;
  break;
case 3:
  P3=(uchar)d;
  break;
case 4:
  P4=(uchar)d;
  break;
case 5:
  P5=(uchar)d;
  break;
default:
  break;
  }
return(0);
}

static entry *m_port_inp(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PI, T_DEZ, 0, 5, T_NONE, 0, 0 };
uint		p;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
switch (p) {
case 0:
  rs232_putx(P0,FALSE);
  break;
case 1:
  rs232_putx(P1,FALSE);
  break;
case 2:
  rs232_putx(P2,FALSE);
  break;
case 3:
  rs232_putx(P3,FALSE);
  break;
case 4:
  rs232_putx(P4,FALSE);
  break;
case 5:
  rs232_putx(P5,FALSE);
  break;
default:
  break;
  }
return(0);
}

static entry *m_an_inp(bit doit) {
/*------------------------------*/
entry	code	e = { ID_AI, T_DEZ, 0, 7, T_HEX, 0x00, 0xFF };
uint		a,r;

if (!doit) {
  if (cmd_c1=='A' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
r=*((uint *)cmd_act);
cmd_act+=2;
ADCON&=0xF0;
ADCON|=a&0x7;
DAPR=r;
while (BSY)
  if (RI)
    break;
putchar('\n');
rs232_putd(ADDAT);
return(0);
}

/* === Ende =========================================================== */
