/************************************************************************
 *  M80C515A.C								*
 *			Befehle fr Prozessor 80C515A fr X51		*
 *			=====================================		*
 *									*
 *  22.06.93	Bodo Wenzel	V1.0, Erstellung			*
 *  02.12.98	Bodo Wenzel	V1.1, Textausgabe krzer		*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <reg515a.h>
#include <stdio.h>

#include "utility.h"
#include "command.h"
#include "m_all.h"
#include "m80c515a.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_PO=ID_SPECIAL, ID_PI, ID_AI, ID_XE, ID_0I, ID_0S, ID_0D };

/* === Funktionsprototypen ============================================ */

static entry *m_port_out(bit doit);
static entry *m_port_inp(bit doit);
static entry *m_an_inp(bit doit);
static entry *m_x_enable(bit doit);
static entry *m_off_idle(bit doit);
static entry *m_off_stop(bit doit);
static entry *m_off_slow(bit doit);

/* === Funktionen ===================================================== */

bit m80c515a_look(void) {
/*---------------------*/
if ((cmd_entry=m_port_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_port_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_an_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_enable(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_idle(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_stop(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_slow(FALSE))!=0)
  return(TRUE);
return(FALSE);
}

void m80c515a_doit(uchar id) {
/*--------------------------*/
switch (id) {
case ID_PO:
  m_port_out(TRUE);
  break;
case ID_PI:
  m_port_inp(TRUE);
  break;
case ID_AI:
  m_an_inp(TRUE);
  break;
case ID_XE:
  m_x_enable(TRUE);
  break;
case ID_0I:
  m_off_idle(TRUE);
  break;
case ID_0S:
  m_off_stop(TRUE);
  break;
case ID_0D:
  m_off_slow(TRUE);
  break;
default:
  break;
  }
}

void m80c515a_help(void) {
/*----------------------*/
rs232_puts("\nPOp,d write port (p=port, d=data)"
           "\nPIp   read port (p=port)"
           "\nAIa   read analog (a=inp)"
           "\nXE    enable xram"
           "\n0I    idle mode (any key wakes up)"
           "\n0S    power down mode (reset wakes up)"
           "\n0De   slow down (e=1) or normal (e=0) with reset");
}

void m80c515a_init(void) {
/*----------------------*/
WDTREL=0x00;			/* Watchdog setzen			*/

if (IP0&0x40) {			/* wenn Watchdog gestartet		*/
  CRCH=(-40000)>>8;
  CRCL=(-40000)&0xFF;
  T2PS=FALSE;
  T2R1=TRUE;
  T2R0=FALSE;
  T2I1=FALSE;
  T2I0=TRUE;
  TF2=FALSE;
  ET2=TRUE;
  }				/* Timer-2-Interrupt freigeben		*/
}

/* --- Watchdog ------------------------------------------------------- */

static void timer2_int(void) interrupt 5 {
/*--------------------------------------*/
TF2=FALSE;

WDT=TRUE;
SWDT=TRUE;			/* Watchdog zurcksetzen		*/
}

/* --- Port-Befehle --------------------------------------------------- */

static entry *m_port_out(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PO, T_DEZ, 0, 5, T_HEX, 0x00, 0xFF };
uint		p,d;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
switch (p) {
case 0:
  P0=(uchar)d;
  break;
case 1:
  P1=(uchar)d;
  break;
case 2:
  P2=(uchar)d;
  break;
case 3:
  P3=(uchar)d;
  break;
case 4:
  P4=(uchar)d;
  break;
case 5:
  P5=(uchar)d;
  break;
default:
  break;
  }
return(0);
}

static entry *m_port_inp(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PI, T_DEZ, 0, 6, T_NONE, 0, 0 };
uint		p;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
switch (p) {
case 0:
  rs232_putx(P0,FALSE);
  break;
case 1:
  rs232_putx(P1,FALSE);
  break;
case 2:
  rs232_putx(P2,FALSE);
  break;
case 3:
  rs232_putx(P3,FALSE);
  break;
case 4:
  rs232_putx(P4,FALSE);
  break;
case 5:
  rs232_putx(P5,FALSE);
  break;
case 6:
  rs232_putx(P6,FALSE);
  break;
default:
  break;
  }
return(0);
}

static entry *m_an_inp(bit doit) {
/*------------------------------*/
entry	code	e = { ID_AI, T_DEZ, 0, 7, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='A' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
ADCON1=0x80;
ADCON0&=0xC0;
ADCON0|=a&0x7;
ADDATL=0;
while (BSY)
  if (RI)
    break;
a=(uint)ADDATH<<2 | (ADDATL>>6);
putchar('\n');
rs232_putd(a);
return(0);
}

static entry *m_x_enable(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_XE, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='E')
    return(&e);
  else
    return(0);
  }
SYSCON&=0xFC;
return(0);
}

static entry *m_off_idle(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0I, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
rs232_puts("CPU is idle");
rs232_int_on();
PCON|=0x01;
PCON|=0x20;
rs232_int_off();
rs232_puts(", waked up");
return(0);
}

static entry *m_off_stop(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0S, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
rs232_puts("power is down\xFF");
PCON|=0x02;
PCON|=0x40;
return(0);
}

static entry *m_off_slow(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0D, T_DEZ, 0, 1, T_NONE, 0, 0 };
uint		en;

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='D')
    return(&e);
  else
    return(0);
  }
en=*((uint *)cmd_act);
cmd_act+=2;
rs232_puts("speed now ");
if (en==0) {
  rs232_puts("normal\xFF");
  PCON&=0xEF;
} else {
  rs232_puts("slow\xFF");
  PCON|=0x10;
  }
EAL=FALSE;
(* (void (*)()) 0U) ();
return(0);
}

/* === Ende =========================================================== */
