/************************************************************************
 *  M80C552.C								*
 *			Befehle fr Prozessor 80C552 fr X51		*
 *			====================================		*
 *									*
 *  22.06.93	Bodo Wenzel	V1.0, Erstellung			*
 *  02.12.98	Bodo Wenzel	V1.1, kleine Fehler beseitigt,		*
 *				Textausgabe krzer			*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <reg552.h>
#include <stdio.h>

#include "utility.h"
#include "command.h"
#include "m_all.h"
#include "m80c552.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_PO=ID_SPECIAL, ID_PI,
	ID_AP, ID_AO, ID_AI,
	ID_IC, ID_IA, ID_I1, ID_I2, ID_IR, ID_IS,
	ID_0I, ID_0S };

static	uchar	i2c_addr;

/* === Funktionsprototypen ============================================ */

static entry *m_port_out(bit doit);
static entry *m_port_inp(bit doit);
static entry *m_an_presc(bit doit);
static entry *m_an_out(bit doit);
static entry *m_an_inp(bit doit);
static entry *m_i2c_clock(bit doit);
static entry *m_i2c_addr(bit doit);
static entry *m_i2c_m_tx1(bit doit);
static entry *m_i2c_m_tx2(bit doit);
static entry *m_i2c_m_rx(bit doit);
static entry *m_i2c_s_txrx(bit doit);
static entry *m_off_idle(bit doit);
static entry *m_off_stop(bit doit);

static void i2c_init(uchar clock);
static uchar i2c_wait(void);
static void i2c_error(uchar err,char code *s);
static void i2c_stop(void);

/* === Funktionen ===================================================== */

bit m80c552_look(void) {
/*--------------------*/
if ((cmd_entry=m_port_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_port_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_an_presc(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_an_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_an_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_clock(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_addr(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_m_tx1(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_m_tx2(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_m_rx(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_i2c_s_txrx(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_idle(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_off_stop(FALSE))!=0)
  return(TRUE);
return(FALSE);
}

void m80c552_doit(uchar id) {
/*-------------------------*/
switch (id) {
case ID_PO:
  m_port_out(TRUE);
  break;
case ID_PI:
  m_port_inp(TRUE);
  break;
case ID_AP:
  m_an_presc(TRUE);
  break;
case ID_AO:
  m_an_out(TRUE);
  break;
case ID_AI:
  m_an_inp(TRUE);
  break;
case ID_IC:
  m_i2c_clock(TRUE);
  break;
case ID_IA:
  m_i2c_addr(TRUE);
  break;
case ID_I1:
  m_i2c_m_tx1(TRUE);
  break;
case ID_I2:
  m_i2c_m_tx2(TRUE);
  break;
case ID_IR:
  m_i2c_m_rx(TRUE);
  break;
case ID_IS:
  m_i2c_s_txrx(TRUE);
  break;
case ID_0I:
  m_off_idle(TRUE);
  break;
case ID_0S:
  m_off_stop(TRUE);
  break;
default:
  break;
  }
}

void m80c552_help(void) {
/*---------------------*/
rs232_puts("\nPOp,d write port (p=port, d=data)"
           "\nPIp   read port (p=port)"
           "\nAPp   pwm prescaler (p=value)"
           "\nAOr,r pwm output (r=ratio)"
           "\nAIa   read analog (a=inp)"
           "\nICc   set I2C clock (c=clock rate)"
           "\nIAa   set address for I1, I2 and IR (a=address)"
           "\nI1d   transmit as master 1 byte (d=data)"
           "\nI2d,d transmit as master 2 bytes (d=data)"
           "\nIR    receive as master"
           "\nISa,d transmit/receive as slave (a=address, d=data)"
           "\n0I    idle mode (any key wakes up)"
           "\n0S    power down mode (reset wakes up)");
}

void m80c552_init(void) {
/*---------------------*/
TM2CON=0x89;			/* fosc/12, Teiler=4, 16-Bit-Interrupt	*/
ET2=TRUE;			/* Timer-2-Interrupt freigeben		*/
}

/* --- Watchdog ------------------------------------------------------- */

static void timer2_int(void) interrupt 14 {
/*---------------------------------------*/
T2OV=FALSE;

PCON|=0x10;
T3=0;				/* Watchdog zurcksetzen		*/
}

/* --- Port-Befehle --------------------------------------------------- */

static entry *m_port_out(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PO, T_DEZ, 0, 4, T_HEX, 0x00, 0xFF };
uint		p,d;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
switch (p) {
case 0:
  P0=(uchar)d;
  break;
case 1:
  P1=(uchar)d;
  break;
case 2:
  P2=(uchar)d;
  break;
case 3:
  P3=(uchar)d;
  break;
case 4:
  P4=(uchar)d;
  break;
default:
  break;
  }
return(0);
}

static entry *m_port_inp(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_PI, T_DEZ, 0, 5, T_NONE, 0, 0 };
uint		p;

if (!doit) {
  if (cmd_c1=='P' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
switch (p) {
case 0:
  rs232_putx(P0,FALSE);
  break;
case 1:
  rs232_putx(P1,FALSE);
  break;
case 2:
  rs232_putx(P2,FALSE);
  break;
case 3:
  rs232_putx(P3,FALSE);
  break;
case 4:
  rs232_putx(P4,FALSE);
  break;
case 5:
  rs232_putx(P5,FALSE);
  break;
default:
  break;
  }
return(0);
}

static entry *m_an_presc(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_AP, T_HEX, 0, 0xFF, T_NONE, 0, 0 };
uint		p;

if (!doit) {
  if (cmd_c1=='A' && cmd_c2=='P')
    return(&e);
  else
    return(0);
  }
p=*((uint *)cmd_act);
cmd_act+=2;
PWMP=(uchar)p;
return(0);
}

static entry *m_an_out(bit doit) {
/*------------------------------*/
entry	code	e = { ID_AO, T_HEX, 0, 0xFF, T_HEX, 0, 0xFF };
uint		r1,r2;

if (!doit) {
  if (cmd_c1=='A' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
r1=*((uint *)cmd_act);
cmd_act+=2;
r2=*((uint *)cmd_act);
cmd_act+=2;
PWM0=(uchar)r1;
PWM1=(uchar)r2;
return(0);
}

static entry *m_an_inp(bit doit) {
/*------------------------------*/
entry	code	e = { ID_AI, T_HEX, 0, 7, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='A' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
ADCON=(uchar)a&0x07;
ADCON|=0x08;
while (!(ADCON&0x10))
  if (RI)
    break;
a=(uint)ADCH<<2 | (ADCON>>6);
ADCON=0x00;
putchar('\n');
rs232_putd(a);
return(0);
}

static entry *m_i2c_clock(bit doit) {
/*---------------------------------*/
entry	code	e = { ID_IC, T_DEZ, 0, 6, T_NONE, 0, 0 };
uint		c;

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='C')
    return(&e);
  else
    return(0);
  }
c=*((uint *)cmd_act);
cmd_act+=2;
i2c_init(c);
return(0);
}

static entry *m_i2c_addr(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_IA, T_HEX, 0x10, 0xEF, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='A')
    return(&e);
  else
    return(0);
  }
i2c_addr=*((uint *)cmd_act);
cmd_act+=2;
return(0);
}

static entry *m_i2c_m_tx1(bit doit) {
/*---------------------------------*/
entry	code	e = { ID_I1, T_HEX, 0x00, 0xFF, T_NONE, 0, 0 };
uint		d;
uchar		r;

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='1')
    return(&e);
  else
    return(0);
  }
d=*((uint *)cmd_act);
cmd_act+=2;

STO=FALSE;
STA=TRUE;
SI=FALSE;
if ((r=i2c_wait())!=0x08) {
  i2c_error(r,"start");
  return(0);
  }
STA=FALSE;
S1DAT=i2c_addr&0xFE;
SI=FALSE;
if ((r=i2c_wait())!=0x18) {
  i2c_error(r,"address");
  return(0);
  }
S1DAT=d;
SI=FALSE;
if ((r=i2c_wait())!=0x28) {
  i2c_error(r,"data");
  return(0);
  }
i2c_stop();
return(0);
}

static entry *m_i2c_m_tx2(bit doit) {
/*---------------------------------*/
entry	code	e = { ID_I2, T_HEX, 0x00, 0xFF, T_HEX, 0x00, 0xFF };
uint		d1,d2;
uchar		r;

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='2')
    return(&e);
  else
    return(0);
  }
d1=*((uint *)cmd_act);
cmd_act+=2;
d2=*((uint *)cmd_act);
cmd_act+=2;

STO=FALSE;
STA=TRUE;
SI=FALSE;
if ((r=i2c_wait())!=0x08) {
  i2c_error(r,"start");
  return(0);
  }
STA=FALSE;
S1DAT=i2c_addr&0xFE;
SI=FALSE;
if ((r=i2c_wait())!=0x18) {
  i2c_error(r,"address");
  return(0);
  }
S1DAT=d1;
SI=FALSE;
if ((r=i2c_wait())!=0x28) {
  i2c_error(r,"data 1");
  return(0);
  }
S1DAT=d2;
SI=FALSE;
if ((r=i2c_wait())!=0x28) {
  i2c_error(r,"data 2");
  return(0);
  }
i2c_stop();
return(0);
}

static entry *m_i2c_m_rx(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_IR, T_NONE, 0, 0, T_NONE, 0, 0 };
uchar		r;

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='R')
    return(&e);
  else
    return(0);
  }

STO=FALSE;
STA=TRUE;
SI=FALSE;
if ((r=i2c_wait())!=0x08) {
  i2c_error(r,"start");
  return(0);
  }
STA=FALSE;
S1DAT=i2c_addr|0x01;
SI=FALSE;
if ((r=i2c_wait())!=0x40) {
  i2c_error(r,"address");
  return(0);
  }
rs232_puts("key at '?': .=\"no ack\" ESC=\"anti *\"\n");
for (;;) {
  putchar('?');
  r=_getkey();
  if (r=='\x1B')
    cmd_repeat=FALSE;
  if (r=='\x1B' || r=='.') {
    AA=FALSE;
    SI=FALSE;
    if ((r=i2c_wait())!=0x58) {
      i2c_error(r,"data");
      return(0);
      }
    rs232_putx(S1DAT,FALSE);
    break;
    }
  AA=TRUE;
  SI=FALSE;
  if ((r=i2c_wait())!=0x50) {
    i2c_error(r,"data");
    return(0);
    }
  rs232_putx(S1DAT,FALSE);
  }
i2c_stop();
return(0);
}

static entry *m_i2c_s_txrx(bit doit) {
/*----------------------------------*/
entry	code	e = { ID_IS, T_HEX, 0x10, 0xEF, T_HEX, 0x00, 0xFF };
uint		a,d;
uchar		r;

if (!doit) {
  if (cmd_c1=='I' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;

rs232_puts("if receiving ('R'), key at '?': "
           "'.'=\"no ack\" ESC=\"anti *\"\n");
S1ADR=a&0xFE;
STO=FALSE;
STA=FALSE;
for (;;) {
  AA=TRUE;
  SI=FALSE;
  switch (r=i2c_wait()) {
  case 0x60:
    putchar('R');
    while (r!=0x00) {
      putchar('?');
      r=_getkey();
      if (r=='\x1B')
        cmd_repeat=FALSE;
      AA=(r!='\x1B' && r!='.');
      SI=FALSE;
      switch (r=i2c_wait()) {
      case 0x80:
      case 0x88:
        rs232_putx(S1DAT,FALSE);
        break;
      case 0xA0:
        putchar('.');
        r=0x00;
        break;
      default:
        i2c_error(r,"data");
        return(0);
        }
      }
    break;
  case 0xA8:
    putchar('T');
    while (r!=0x00) {
      putchar('t');
      S1DAT=d;
      SI=FALSE;
      switch (r=i2c_wait()) {
      case 0xB8:
        break;
      case 0xC0:
        putchar('.');
        r=0x00;
        break;
      default:
        i2c_error(r,"data");
        return(0);
        }
      }
    break;
  case 0xFF:
    return(0);
  default:
    i2c_error(r,"address");
    return(0);
    }
  }
return(0);
}

static entry *m_off_idle(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0I, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
rs232_puts("CPU is idle");
rs232_int_on();
PCON|=0x01;
rs232_int_off();
rs232_puts(", waked up");
return(0);
}

static entry *m_off_stop(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_0S, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='0' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
rs232_puts("power is down\xFF");
PCON|=0x02;
return(0);
}

/* === IC-Funktionen ================================================= */

static void i2c_init(uchar clock) {
/*-------------------------------*/
switch (clock) {
case 0:
  S1CON=0x40;
  break;
case 1:
  S1CON=0x41;
  break;
case 2:
  S1CON=0x42;
  break;
case 3:
  S1CON=0x43;
  break;
case 4:
  S1CON=0xC0;
  break;
case 5:
  S1CON=0xC1;
  break;
case 6:
  S1CON=0xC2;
  break;
default:
  break;
  }
SCL=1;
SDA=1;
}

static uchar i2c_wait(void) {
/*-------------------------*/
while (!SI)
  if (RI)
    return(0xFF);
return(S1STA);
}

static void i2c_error(uchar err,char code *s) {
/*-------------------------------------------*/
if (err!=0xFF) {
  rs232_puts("\nerror <");
  rs232_putx(err,FALSE);
  rs232_puts("> at ");
} else
  rs232_puts("\naborted at ");
rs232_puts(s);
i2c_stop();
}

static void i2c_stop(void) {
/*------------------------*/
STA=FALSE;
STO=TRUE;
AA=FALSE;
SI=FALSE;
while (STO)
  if (RI)
    break;
}

/* === Ende =========================================================== */
