/************************************************************************
 *  M_ALL.C								*
 *			Befehle fr alle Prozessoren fr X51		*
 *			====================================		*
 *									*
 *  28.06.93	Bodo Wenzel	V1.0, Erstellung			*
 *  02.12.98	Bodo Wenzel	V1.1, Textausgabe krzer,		*
 *				Speichertestbefehl XC			*
 ************************************************************************/

#pragma code symbols small debug

/* === Einfgedateien ================================================= */

#include <absacc.h>
#include <reg51.h>
#include <stdio.h>

#include "utility.h"
#include "x51.h"
#include "command.h"
#include "m_all.h"
#include "m8051.h"
#include "m80c51.h"
#include "m8052.h"
#include "m80c52.h"
#include "m80515.h"
#include "m80c515.h"
#include "m80c515a.h"
#include "m80c552.h"
#include "m80c652.h"
#include "sfr.h"
#include "rs232.h"

/* === Globale Variablen ============================================== */

enum {	ID_HE=ID_NONE+1, ID_TY, ID_RS, ID_FM,
	ID_SO, ID_SI, ID_SS, ID_ST,
	ID_XW, ID_XR, ID_XO, ID_XI, ID_XP, ID_XT, ID_XC,
	ID_MH, ID_ML, ID_OH, ID_OL };

/* === Funktionsprototypen ============================================ */

static entry *m_help(bit doit);
static entry *m_type(bit doit);
static entry *m_reset(bit doit);
static entry *m_frequ(bit doit);
static entry *m_sfr_out(bit doit);
static entry *m_sfr_inp(bit doit);
static entry *m_sfrb_set(bit doit);
static entry *m_sfrb_test(bit doit);
static entry *m_x_write(bit doit);
static entry *m_x_read(bit doit);
static entry *m_x_out(bit doit);
static entry *m_x_inp(bit doit);
static entry *m_x_prep(bit doit);
static entry *m_x_test(bit doit);
static entry *m_x_check(bit doit);
static bit mxt_check(uint a,uchar w);
static entry *m_meas_h(bit doit);
static entry *m_meas_l(bit doit);
static entry *m_out_h(bit doit);
static entry *m_out_l(bit doit);

/* === Funktionen ===================================================== */

void m_prompt(void) {
/*-----------------*/
switch (cmd_proc) {
case P8051:
  rs232_puts("8051");
  break;
case P80C51:
  rs232_puts("80C51");
  break;
case P8052:
  rs232_puts("8052");
  break;
case P80C52:
  rs232_puts("80C52");
  break;
case P80515:
  rs232_puts("80515");
  break;
case P80C515:
  rs232_puts("80C515");
  break;
case P80C515A:
  rs232_puts("80C515A");
  break;
case P80C552:
  rs232_puts("80C552");
  break;
case P80C652:
  rs232_puts("80C652");
  break;
default:
  rs232_puts("XXXX");
  break;
  }
}

/* --- Kommandoverzweigungen ------------------------------------------ */

bit m_all_look(void) {
/*------------------*/
if ((cmd_entry=m_help(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_type(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_reset(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_frequ(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfr_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfr_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfrb_set(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_sfrb_test(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_write(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_read(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_out(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_inp(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_prep(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_test(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_x_check(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_meas_h(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_meas_l(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_out_h(FALSE))!=0)
  return(TRUE);
if ((cmd_entry=m_out_l(FALSE))!=0)
  return(TRUE);

switch (cmd_proc) {
case P8051:
  return(m8051_look());
case P80C51:
  return(m80c51_look());
case P8052:
  return(m8052_look());
case P80C52:
  return(m80c52_look());
case P80515:
  return(m80515_look());
case P80C515:
  return(m80c515_look());
case P80C515A:
  return(m80c515a_look());
case P80C552:
  return(m80c552_look());
case P80C652:
  return(m80c652_look());
default:
  return(FALSE);
  }
}

void m_all_doit(void) {
/*-------------------*/
uchar	id;

switch (id=*cmd_act++) {
case ID_HE:
  m_help(TRUE);
  break;
case ID_TY:
  m_type(TRUE);
  break;
case ID_RS:
  m_reset(TRUE);
  break;
case ID_FM:
  m_frequ(TRUE);
  break;
case ID_SO:
  m_sfr_out(TRUE);
  break;
case ID_SI:
  m_sfr_inp(TRUE);
  break;
case ID_SS:
  m_sfrb_set(TRUE);
  break;
case ID_ST:
  m_sfrb_test(TRUE);
  break;
case ID_XW:
  m_x_write(TRUE);
  break;
case ID_XR:
  m_x_read(TRUE);
  break;
case ID_XO:
  m_x_out(TRUE);
  break;
case ID_XI:
  m_x_inp(TRUE);
  break;
case ID_XP:
  m_x_prep(TRUE);
  break;
case ID_XT:
  m_x_test(TRUE);
  break;
case ID_XC:
  m_x_check(TRUE);
  break;
case ID_MH:
  m_meas_h(TRUE);
  break;
case ID_ML:
  m_meas_l(TRUE);
  break;
case ID_OH:
  m_out_h(TRUE);
  break;
case ID_OL:
  m_out_l(TRUE);
  break;
default:
  switch (cmd_proc) {
  case P8051:
    m8051_doit(id);
    break;
  case P80C51:
    m80c51_doit(id);
    break;
  case P8052:
    m8052_doit(id);
    break;
  case P80C52:
    m80c52_doit(id);
    break;
  case P80515:
    m80515_doit(id);
    break;
  case P80C515:
    m80c515_doit(id);
    break;
  case P80C515A:
    m80c515a_doit(id);
    break;
  case P80C552:
    m80c552_doit(id);
    break;
  case P80C652:
    m80c652_doit(id);
    break;
  default:
    break;
    }
  }
}

/* --- Hilfe ausgeben ------------------------------------------------- */

static entry *m_help(bit doit) {
/*----------------------------*/
entry	code	e = { ID_HE, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='H' && cmd_c2=='E')
    return(&e);
  else
    return(0);
  }
rs232_puts(
"\nEmpty inputs repeats last command."
"\nAsterisk as first char makes command looping."
"\nESC aborts entry, comma separates values, point separates commands."
"\nAvailable commands:"
"\nHE    shows this"
"\nTYt   select type (0=8051, 1=80C51, 2=8052, 3=80C52,"
"\n      4=80515, 5=80C515, 6=80C515A, 7=80C552, 8=80C652)"
"\nRS    soft reset"
"\nFMb   crystal frequency (b=baudrate)"
"\nSOa,d write sfr byte (a=addr, d=data)"
"\nSIa   read sfr byte (a=addr)"
"\nSSa,d set sfr bit (a=addr, d=data)"
"\nSTa   test sfr bit (a=addr)"
"\nPress any key to continue, ESC to abort..."
);
if (_getkey()==0x1B)
  return(0);
rs232_puts(
"\nXWa,d write external memory, autoinc (a=addr, d=data)"
"\nXRa   read external memory, autoinc (a=addr)"
"\nXOa,d write external memory (a=addr, d=data)"
"\nXIa   read external memory (a=addr)"
"\nXPb,e prepare external memory for test (b=begin, e=end)"
"\nXTb,e test external memory (b=begin, e=end)"
"\nXCn,o check external memory hardware (2^n bytes at o*2^n)"
"\nMHa   measure high pulse (a=sfr bit addr)"
"\nMLa   measure low pulse (a=sfr bit addr)"
"\nOHa,t high pulse output (a=sfr bit addr, t=ticks)"
"\nOLa,t low pulse output (a=sfr bit addr, t=ticks)"
"\nPress any key to continue, ESC to abort..."
);
if (_getkey()==0x1B)
  return(0);

rs232_puts("\nSpecial commands for selected type:");
switch (cmd_proc) {
case P8051:
  m8051_help();
  break;
case P80C51:
  m80c51_help();
  break;
case P8052:
  m8052_help();
  break;
case P80C52:
  m80c52_help();
  break;
case P80515:
  m80515_help();
  break;
case P80C515:
  m80c515_help();
  break;
case P80C515A:
  m80c515a_help();
  break;
case P80C552:
  m80c552_help();
  break;
case P80C652:
  m80c652_help();
  break;
default:
  break;
  }
return(0);
}

/* --- Typ auswhlen  ------------------------------------------------- */

static entry *m_type(bit doit) {
/*----------------------------*/
entry	code	e = { ID_TY, T_DEZ, 0, PMAX-1, T_NONE, 0, 0 };
uint		t;

if (!doit) {
  if (cmd_c1=='T' && cmd_c2=='Y')
    return(&e);
  else
    return(0);
  }
t=*((uint *)cmd_act);
cmd_act+=2;
switch (cmd_proc=(uchar)t) {
case P8051:
  m8051_init();
  break;
case P80C51:
  m80c51_init();
  break;
case P8052:
  m8052_init();
  break;
case P80C52:
  m80c52_init();
  break;
case P80515:
  m80515_init();
  break;
case P80C515:
  m80c515_init();
  break;
case P80C515A:
  m80c515a_init();
  break;
case P80C552:
  m80c552_init();
  break;
case P80C652:
  m80c652_init();
  break;
default:
  break;
  }
return(0);
}

/* --- Allgemeine Befehle --------------------------------------------- */

static entry *m_reset(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_RS, T_NONE, 0, 0, T_NONE, 0, 0 };

if (!doit) {
  if (cmd_c1=='R' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
putchar('\n');
putchar('\xFF');
EA=FALSE;
(* (void (*)()) 0U) ();
return(0);
}

static entry *m_frequ(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_FM, T_DEZ, 50, 19200, T_NONE, 0, 0 };
long		f;

if (!doit) {
  if (cmd_c1=='F' && cmd_c2=='M')
    return(&e);
  else
    return(0);
  }
f=*((uint *)cmd_act);
cmd_act+=2;
f*=bit_ticks;
f+=5000L;
putchar('\n');
rs232_putd(f/1000000L);
putchar('.');
f%=1000000L;
rs232_putd(f/100000L);
f%=100000L;
rs232_putd(f/10000L);
return(0);
}

/* --- Spezial-Funktion-Register bearbeiten --------------------------- */

static entry *m_sfr_out(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_SO, T_HEX, 0x80, 0xFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
sfr_out((uchar)a,(uchar)d);
return(0);
}

static entry *m_sfr_inp(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_SI, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putx(sfr_inp((uchar)a),FALSE);
return(0);
}

static entry *m_sfrb_set(bit doit) {
/*--------------------------------*/
entry	code	e = { ID_SS, T_HEX, 0x80, 0xFF, T_DEZ, 0, 1 };
uint		a,d;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='S')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
sfr_set((uchar)a,(bit)d);
return(0);
}

static entry *m_sfrb_test(bit doit) {
/*---------------------------------*/
entry	code	e = { ID_ST, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='S' && cmd_c2=='T')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putd(sfr_test((uchar)a));
return(0);
}

/* --- Externen Speicher bearbeiten ----------------------------------- */

static entry *m_x_write(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_XW, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='W')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_read(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XR, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='R')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
*((uint *)cmd_act)=a+1;
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_out(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XO, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x00, 0xFF };
uint		a,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='O')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
d=*((uint *)cmd_act);
cmd_act+=2;
XBYTE[a]=(uchar)d;
return(0);
}

static entry *m_x_inp(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_XI, T_HEX, 0x0000, 0xFFFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='I')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
putchar('\n');
rs232_putx(XBYTE[a],FALSE);
return(0);
}

static entry *m_x_prep(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XP, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='P')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  XBYTE[a]=d;
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
XBYTE[a]=d;
return(0);
}

static entry *m_x_test(bit doit) {
/*------------------------------*/
entry	code	e = { ID_XT, T_HEX, 0x0000, 0xFFFF, T_HEX, 0x0000, 0xFFFF };
uint		a,a_e;
uchar		d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='T')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
a_e=*((uint *)cmd_act);
cmd_act+=2;
d=0x13;
while (a<a_e) {
  if (!mxt_check(a,d))
    return(0);
  d+=(a>>8)+(uchar)a+9;
  a++;
  }
(void)mxt_check(a,d);
return(0);
}

static entry *m_x_check(bit doit) {
/*-------------------------------*/
entry	code	e = { ID_XC, T_DEZ, 4, 16, T_HEX, 0x000, 0xFFF };
uint		s,a,m;
uchar		err_d,err_al,err_ah,d;

if (!doit) {
  if (cmd_c1=='X' && cmd_c2=='C')
    return(&e);
  else
    return(0);
  }
s=*((uint *)cmd_act);
m=~(0xFFFF<<(uchar)s);
s=m+1;
cmd_act+=2;
a=*((uint *)cmd_act)*s;
cmd_act+=2;
err_d=0x00;
for (d=0x01; d!=0x00; d<<=1) {
  XBYTE[a]=d;
  if ((XBYTE[a]&d)==0)
    err_d|=d;
  XBYTE[a]=~d;
  if ((XBYTE[a]&d)!=0)
    err_d|=d;
  }
putchar('\n');
if (err_d==0x00) {
  err_al=0x00;
  for (d=0x01; d!=(s&0xFF); d<<=1) {
    XBYTE[a]=0x55;
    XBYTE[a^d]=d;
    if (XBYTE[a]!=0x55)
      err_al|=d;
    XBYTE[a|m]=0xAA;
    XBYTE[(a|m)^d]=d;
    if (XBYTE[a|m]!=0xAA)
      err_al|=d;
    }
  err_ah=0x00;
  if ((s&0xFF)==0x00)
    for (d=0x01; d!=(s>>8); d<<=1) {
      XBYTE[a]=0x55;
      XBYTE[a^((uint)d<<8)]=d;
      if (XBYTE[a]!=0x55)
        err_ah|=d;
      XBYTE[a|m]=0xAA;
      XBYTE[(a|m)^((uint)d<<8)]=d;
      if (XBYTE[a|m]!=0xAA)
        err_ah|=d;
      }
  rs232_putx(err_ah,FALSE);
  rs232_putx(err_al,FALSE);
} else
  rs232_puts("XXXX");
putchar('-');
rs232_putx(err_d,FALSE);
return(0);
}

static bit mxt_check(uint a,uchar w) {
/*----------------------------------*/
uchar	r;

r=XBYTE[a];
if (r==w)
  return(TRUE);

putchar('\n');
rs232_putx(a,TRUE);
putchar(':');
rs232_putx(w,FALSE);
putchar('>');
rs232_putx(r,FALSE);
rs232_puts(" press any key, '.' aborts ESC=\"anti *\"");
r=_getkey();
if (r=='\x1B')
  cmd_repeat=FALSE;
return(r!='\x1B' && r!='.');
}

/* --- Peripherie-Tests ----------------------------------------------- */

static entry *m_meas_h(bit doit) {
/*------------------------------*/
entry	code	e = { ID_MH, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='M' && cmd_c2=='H')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=0;
TL0=0;
TF0=FALSE;
while (!sfr_test((uchar)a))
  if (RI)
    break;
TR0=TRUE;
while (sfr_test((uchar)a))
  if (RI)
    break;
TR0=FALSE;
putchar('\n');
if (!TF0) {
  a=((uint)TH0<<8) | TL0;
  rs232_putd(a);
} else
  rs232_puts("Overflow");
if (RI)
  _getkey();
return(0);
}

static entry *m_meas_l(bit doit) {
/*------------------------------*/
entry	code	e = { ID_ML, T_HEX, 0x80, 0xFF, T_NONE, 0, 0 };
uint		a;

if (!doit) {
  if (cmd_c1=='M' && cmd_c2=='L')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=0;
TL0=0;
TF0=FALSE;
while (sfr_test((uchar)a))
  if (RI)
    break;
TR0=TRUE;
while (!sfr_test((uchar)a))
  if (RI)
    break;
TR0=FALSE;
putchar('\n');
if (!TF0) {
  a=((uint)TH0<<8) | TL0;
  rs232_putd(a);
} else
  rs232_puts("Overflow");
if (RI)
  _getkey();
return(0);
}

static entry *m_out_h(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_OH, T_HEX, 0x80, 0xFF, T_DEZ, 0, 65535 };
uint		a,t;

if (!doit) {
  if (cmd_c1=='O' && cmd_c2=='H')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
t=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=(-(int)t)>>8;
TL0=(uchar)(-(int)t);
TF0=FALSE;
TR0=TRUE;
sfr_set((uchar)a,1);
while (!TF0);
sfr_set((uchar)a,0);
return(0);
}

static entry *m_out_l(bit doit) {
/*-----------------------------*/
entry	code	e = { ID_OL, T_HEX, 0x80, 0xFF, T_DEZ, 0, 65535 };
uint		a,t;

if (!doit) {
  if (cmd_c1=='O' && cmd_c2=='L')
    return(&e);
  else
    return(0);
  }
a=*((uint *)cmd_act);
cmd_act+=2;
t=*((uint *)cmd_act);
cmd_act+=2;
TR0=FALSE;
TMOD&=0xF0;
TMOD|=0x01;
TH0=(-(int)t)>>8;
TL0=(uchar)(-(int)t);
TF0=FALSE;
TR0=TRUE;
sfr_set((uchar)a,0);
while (!TF0);
sfr_set((uchar)a,1);
return(0);
}

/* === Ende =========================================================== */
